import argparse
import logging
import socket
import struct

if not hasattr(socket, 'SO_BINDTODEVICE'):
    socket.SO_BINDTODEVICE = 25

hostname = socket.gethostname()
logging.basicConfig(filename=hostname+'-client6.log', encoding='utf-8', format='%(asctime)s %(message)s', level=logging.DEBUG)

parser = argparse.ArgumentParser()
parser.add_argument("multicast_group", help="Multicast IPv6 to use for subscription")

args = parser.parse_args()
server_address = ('', 10000)

sock = socket.socket(socket.AF_INET6, socket.SOCK_DGRAM)
sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
sock.bind(server_address)

# on all interfaces.
for ifidx, ifname in socket.if_nameindex():
    if ifname in ('lo', 'can0'):
        continue
    logging.info('%s joining %s @ [%d]%s', hostname, args.multicast_group, ifidx, ifname)
    bifidx = struct.pack("@i", ifidx)
    mreq = socket.inet_pton(socket.AF_INET6, args.multicast_group) + bifidx
    sock.setsockopt(socket.IPPROTO_IPV6, socket.IPV6_JOIN_GROUP, mreq)

try:
    while True:
        data, address = sock.recvfrom(10240)
        logging.info('%s received %s', hostname, data.decode("utf-8"))
finally:
    sock.close()
